'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var repositoryProvider = require('repository-provider');
var fs = require('fs');
var util = require('util');
var path = require('path');
var os = require('os');

const execa = require('execa');

const pStat = util.promisify(fs.stat);

/**
 * @property {string} workspace
 */
class LocalRepository extends repositoryProvider.Repository {
  /**
   * exec git clone or git pull
   * @param {string} workspace
   */
  async initialize(workspace) {
    Object.defineProperty(this, 'workspace', { value: workspace });
    await super.initialize();
    try {
      await pStat(this.workspace);
      const result = await execa('git', ['pull'], { cwd: this.workspace });
    } catch (e) {
      if (e.code === 'ENOENT') {
        const result = await execa('git', ['clone', this.name, this.workspace]);
      } else {
        throw e;
      }
    }
    await this.initializeBranches();
  }

  async initializeBranches() {
    const result = await execa('git', ['branch'], {
      cwd: this.workspace
    });

    result.stdout.split(/\n/).forEach(b => {
      const m = b.match(/^\*?\s*([^\s]+)/);
      if (m) {
        const name = m[1];
        const branch = new this.provider.branchClass(this, name);
        this._branches.set(branch.name, branch);
      }
    });
  }

  get urls() {
    return [this.name];
  }

  async push() {
    return execa('git', ['push'], {
      cwd: this.workspace
    });
  }

  async createBranch(name, from) {
    const result = await execa('git', ['checkout', '-b', name], {
      cwd: this.workspace
    });

    const b = new this.provider.branchClass(this, name);
    this._branches.set(b.name, b);
    return b;
  }

  async deleteBranch(name) {
    const options = {
      cwd: this.workspace
    };

    await execa('git', ['checkout', 'master'], options);
    const result = await execa('git', ['branch', '-D', name], options);

    this._branches.delete(name);
  }
}

const execa$1 = require('execa');
const globby = require('globby');
const makeDir = require('make-dir');

const pReadFile = util.promisify(fs.readFile);
const pWriteFile = util.promisify(fs.writeFile);

/**
 * @property {string} workspace
 */
class LocalBranch extends repositoryProvider.Branch {
  get workspace() {
    return this.repository.workspace;
  }

  async content(fileName, options = {}) {
    try {
      const d = pReadFile(path.join(this.workspace, fileName), {
        encoding: 'utf8'
      });

      return new repositoryProvider.Content(fileName, await d);
    } catch (e) {
      if (options.ignoreMissing) {
        return new repositoryProvider.Content(fileName, '');
      }
      throw e;
    }
  }

  /**
   * Excutes:
   * - writes all updates into the workspace
   * - git add
   * - git commit
   * - git push
   * @param {string} message commit message
   * @param {Content[]} updates file content to be commited
   * @param {Object} options
   */
  async commit(message, updates, options = {}) {
    await Promise.all(
      updates.map(b => makeDir(path.dirname(path.join(this.workspace, b.path))))
    );

    await Promise.all(
      updates.map(b => pWriteFile(path.join(this.workspace, b.path), b.content))
    );

    const execaOptions = {
      cwd: this.workspace
    };

    await execa$1('git', ['add', ...updates.map(b => b.path)], execaOptions);
    await execa$1('git', ['commit', '-m', message], execaOptions);
    await execa$1(
      'git',
      ['push', '--set-upstream', 'origin', this.name],
      execaOptions
    );
  }

  async list() {
    return (await globby(['**/.*', '**/*'], { cwd: this.workspace })).map(f => {
      return { path: f, type: 'blob' };
    });
  }

  async createPullRequest(to, message) {
    return new this.provider.pullRequestClass(this.repository, '0', {
      title: 'plese create pull request manually'
    });
  }
}

const pstat = util.promisify(fs.stat);

/**
 * Provider using native git executable
 */
class LocalProvider extends repositoryProvider.Provider {
  /**
   * Default configuration options
   * - workspace
   * @return {Object}
   */
  static get defaultOptions() {
    return { workspace: os.tmpdir() };
  }

  /**
   * @return {Object} empty object
   */
  static optionsFromEnvironment(env) {
    return {};
  }

  get workspace() {
    return this.config.workspace;
  }

  get repositoryClass() {
    return LocalRepository;
  }

  get branchClass() {
    return LocalBranch;
  }

  /**
   * Generate path for a new workspace
   * For the livetime of the provider always genrate new names
   * @return {string} path
   */
  async newWorkspacePath() {
    do {
      this._nextWorkspace =
        this._nextWorkspace === undefined ? 1 : this._nextWorkspace + 1;

      let w = path.join(this.workspace, `r${this._nextWorkspace}`);
      try {
        const s = await pstat(w);
      } catch (e) {
        return w;
      }
    } while (true);
  }

  /**
   * using provider workspace and number of repositories to create repository workspace
   * @param {string} name
   */
  async repository(name) {
    let r = this.repositories.get(name);
    if (r === undefined) {
      if (name === undefined) {
        return undefined;
      }

      r = new this.repositoryClass(this, name);
      await r.initialize(await this.newWorkspacePath());
      this.repositories.set(name, r);
    }

    return r;
  }
}

exports.LocalProvider = LocalProvider;
